;; eepitch.el - a refactored version of some parts of eev.
;; This implements a simpler way to interact with external programs
;; than the one shown in:
;;        <http://angg.twu.net/eev-current/anim/channels.anim.html>
;;
;; Author: Eduardo Ochs <eduardoochs@gmail.com>.
;; Version: 2011nov24
;; License: GPL3.
;; Source:        <http://angg.twu.net/eev-current/eepitch.el>
;; Htmlized:      <http://angg.twu.net/eev-current/eepitch.el.html>
;; Documentation: <http://angg.twu.net/eev-current/eepitch.readme.html>
;;
;; This version is flexible enough to cope with Emacs modes that use
;; several windows, like GUD (for gdb and friends) and SLIME (for
;; external Lisps), but the exact details of how to handle them still
;; have to be documented - and the tests need to finished, cleaned up,
;; etc.
;;
;; The current standard way to install eev is through this:
;;   http://angg.twu.net/debian/README.html
;; But this version can be used independently of the rest of eev.
;; The current priorities are:
;;   1) support for GUD and SLIME,
;;   2) finish the docs in plain text format, that are at
;;        http://angg.twu.net/eev-current/eepitch.readme.html
;;   2) write the docs for eepitch in blogme4/texinfo - like this,
;;        http://angg.twu.net/blogme4/doc/
;;      but in a way that makes it clear how to use this indepently
;;      from the rest,
;;   3) a debian package - "eev-puro" - with the demo scripts that a
;;      bunch of students from PURO are using to learn Lua (and *NIX).
;;      See:
;;        http://angg.twu.net/eev-puro.html
;;        http://angg.twu.net/eev-puro/
;;      It works well, but I want to have more students running it...
;;      It needs better examples and needs to be translated to English.
;;   4) compare this with org-babel and org-babel-screen
;;   5) compare eev-puro with emacs-starter-kit

;; As this is still a bit prototype-ish we set two keybindings and a
;; glyph globally - search for "set-glyph" and "global-set-key" below.
;; A basic test:
;;   (load "eepitch.el")
;; Then in the block below type M-T on the `shell' line,
;; then use `F8's.
'(

shell
cd /tmp/
ls

)

;; In order to understand how this works, let's follow an example and
;; make some definitions. I will suppose that you know how to use
;; eepitch in basic cases. Consider this Emacs frame:
;;                                 ___
;;    ______________emacs_________|-|X|
;;   |                                 |  \
;;   |  (eepitch '(shell))            |  | Let's call this the
;;   | cd /tmp/                        |  | "e-script window".
;;   | ls                              |  | The point is at the "_".
;;   | _                               |  |
;;   |                                 |  /
;;   |--:** NOTES  (Fundamental) ------|  <-- Its modeline.
;;   |                                 |  \
;;   | /home/edrx# cd /tmp/            |  | Let's call this the
;;   | /tmp# ls                        |  | "target window".
;;   | ./  ../  foo.txt                |  |
;;   | /tmp#                           |  |
;;   |                                 |  /
;;   |--:** *shell*  (Shell:run) ------|  <-- Its modeline.
;;   |_________________________________|  <-- The minibuffer.
;;
;; When we typed F8 on the line " (eepitch '(shell))",
;;   a) `eepitch-code' was set to `(shell)',
;;   b) eval'ing `(shell)' created a buffer called "*shell*",
;;   c) `eepitch-buffer-name' was set to the string "*shell*",
;;   d) the buffer "*shell*" was displayed in another window - let's
;;      call it the "target window".
;; The actions b, c and d were performed by `eepitch-prepare'.
;;
;; When we typed F8 on the line "cd /tmp/" the string "cd /tmp/" was
;; "pitched" to the target window, by `(eepitch-line "cd /tmp/")'.
;; Same for the "ls" in the next line.
;;
;; Pitching lines is easy when the system is "prepared". 
;; The tricky part is the definition of "prepared".
;;
;; When
;;   1) `eepitch-target-buffer' holds the name of the buffer opened by
;;      eval'ing the sexp stored in `eepitch-code', and
;;   2) a buffer with that name exists - let's call it the "target
;;      buffer",
;; we say that the system is "(at least) half prepared".
;;
;; When
;;   3) the target buffer is not the current buffer
;; we say that the target is "proper".
;;
;; When 1, 2 and 3 hold and, besides that,
;;   4) there's a window displaying the target buffer,
;; we say that the system is "prepared".
;;
;; `eepitch-buffer-create' takes care of conditions 1 and 2.
;; `eepitch-assert-not-target', of condition 3.
;; `eepitch-window-show', of condition 4 (supposing 1, 2, 3 hold).
;; `eepitch-prepare', of conditions 1-4.

(defun ee-bol () (point-at-bol))
(defun ee-eol () (point-at-eol))
(defun ee-eval-string (str)
  "Wrap STR in a progn then read it and eval it.
Examples: (ee-eval-string \"(+ 1 2) (* 3 4) ;; this returns 12=3*4\")
          (ee-eval-string \";; this returns nil\")"
  (eval (read (concat "(progn\n" str "\n)"))))
(defun ee-next-line (&optional arg try-vscroll)
  (let ((line-move-visual nil))
    (next-line arg try-vscroll)))

(defun ee-kill-buffer (buffer)
  "Kill BUFFER if it exists, asking for fewer confirmations than usual."
  (if (get-buffer buffer)
      (let ((kill-buffer-query-functions nil))
	(kill-buffer buffer))))

(defun eek (str)
  "Execute STR as a keyboard macro. See `edmacro-mode' for the exact format.\n
An example: (eek \"C-x 4 C-h\")"
  (execute-kbd-macro (read-kbd-macro str)))



;;;                     
;;;   ___ ___  _ __ ___ 
;;;  / __/ _ \| '__/ _ \
;;; | (_| (_) | | |  __/
;;;  \___\___/|_|  \___|
;;;                     

(defvar eepitch-regexp "^\\(.*\\)")

(defvar eepitch-buffer-name ""
  "The name of the target buffer for eepitch.
Set this to \"\" to force running `eepitch-buffer-create' again.
Note that `eepitch-buffer-create' sets this variable!")

(defvar eepitch-code '(error "eepitch not set up")
  "The code to create and switch to the target buffer.")

(defvar eepitch-window-show    '(eepitch-window-show))  ; cheap indirection
(defvar eepitch-kill           '(eepitch-kill-buffer))  ; cheap indirection
(defvar eepitch-kill-windows   'nil)	                ; cheap indirection
(defun eepitch-buffer-exists () (get-buffer        eepitch-buffer-name))
(defun eepitch-window-exists () (get-buffer-window eepitch-buffer-name))
(defun eepitch-target-buffer () (get-buffer        eepitch-buffer-name))
(defun eepitch-target-window () (get-buffer-window eepitch-buffer-name))
(defun eepitch-target-here () (eq (current-buffer) (eepitch-target-buffer)))

(defun eepitch-buffer-create ()
  "Eval `eepitch-code' and set `eepitch-buffer-name'.
To understand this function, see `eepitch-prepare'."
  (save-window-excursion
    (eval eepitch-code)
    (setq eepitch-buffer-name
	  (buffer-name (current-buffer)))))

(defun eepitch-assert-not-target ()
  "See `eepitch-prepare'."
  (if (eepitch-target-here) ; improper target?
      (error "Can't pitch to the current buffer")))

(defun eepitch-window-show ()
  "Display the buffer `eepitch-buffer-name' in another window.
This function uses `display-buffer', which calls `split-window-sensibly'."
  (let ((pop-up-windows t)
	(same-window-buffer-names nil))
    (display-buffer eepitch-buffer-name)))

(defun eepitch-prepare ()
"If the eepitch buffer does not exist, create it; if it is not shown, show it.
As a bonus, make sure that the current buffer and the target buffer
for eepitch are different (raise an error if they are the same).

Note that by setting `eepitch-buffer-name' to \"\" we assert that
the target buffer does not exist, and this forces
`eepitch-buffer-create' to run; but `eepitch-buffer-create' sets
`eepitch-buffer-name'... for example, if `eepitch-code'
is (shell) then `eepitch-buffer-name' will be set to \"*shell*\".
So this function does quite a lot."
  (if (not (eepitch-buffer-exists))
      (eepitch-buffer-create))
  (eepitch-assert-not-target)
  (if (not (eepitch-window-exists))
      (eval eepitch-window-show)))

(defun eepitch (code)
  (setq eepitch-code code)
  (setq eepitch-window-show '(eepitch-window-show))
  (setq eepitch-buffer-name "")	; so that `eepitch-buffer-exists' will fail
  (setq eepitch-kill '(eepitch-kill-buffer))
  (eepitch-prepare))

(defun eepitch-line (line)
  (save-selected-window
    (select-window (eepitch-target-window))
    (insert line)                              ; "type" the line
    (call-interactively (key-binding "\r"))))  ; then do a RET

(defun eepitch-this-line ()
"Pitch this line to the target buffer, or eval it as lisp if it starts with `'.
See `eepitch' and the source code."
  (interactive)
  (let ((line (buffer-substring (ee-bol) (ee-eol)))) ; contents of this line
    (if (string-match eepitch-regexp line)           ; a line with a red star
        (ee-eval-string (match-string 1 line))       ; are eval'ed, but
      (eepitch-prepare)		   ; for other lines prepare the target buffer
      (eepitch-line line)))	   ; and pitch the line to the target.
  (ee-next-line 1))

(defun eepitch-kill () (eval eepitch-kill))
(defun eepitch-kill-buffer ()
  (if (eepitch-buffer-exists)
      (if (eepitch-target-here)
	  (error "Can't kill this")
	(ee-kill-buffer eepitch-buffer-name) ; kill with few warnings
	)))

	;; (eval eepitch-kill-windows)



;;;                  _ _       _               _          _ _ 
;;;   ___  ___ _ __ (_) |_ ___| |__        ___| |__   ___| | |
;;;  / _ \/ _ \ '_ \| | __/ __| '_ \ _____/ __| '_ \ / _ \ | |
;;; |  __/  __/ |_) | | || (__| | | |_____\__ \ | | |  __/ | |
;;;  \___|\___| .__/|_|\__\___|_| |_|     |___/_| |_|\___|_|_|
;;;           |_|                                             

(defun eepitch-shell ()
  "Same as (eepitch '(shell)). See `eepitch' and `ee-wrap-eepitch'."
  (interactive)
  (eepitch '(shell)))

(defun eepitch-shell2  () (interactive) (eepitch '(shell "*shell 2*")))
(defun eepitch-eshell  () (interactive) (eepitch '(eshell)))



;;;                      _       _   
;;;   ___ ___  _ __ ___ (_)_ __ | |_ 
;;;  / __/ _ \| '_ ` _ \| | '_ \| __|
;;; | (_| (_) | | | | | | | | | | |_ 
;;;  \___\___/|_| |_| |_|_|_| |_|\__|
;;;                                  

(defun ee-expand (fname)
"Expand \"~\"s and \"$ENVVAR\"s in file names, but only at the beginning of the string."
  (cond ((string-match "^\\$\\([A-Za-z_][0-9A-Za-z_]*\\)\\(.*\\)" fname)
	 (concat (getenv (match-string 1 fname))
		 (match-string 2 fname)))
	((string-match "^\\(~\\([a-z][0-9a-z_]*\\)?\\)\\(/.*\\)?$" fname)
	 (concat (expand-file-name (match-string 1 fname))
		 (match-string 3 fname)))
	(t fname)))

(defun ee-split (str) (if (stringp str) (split-string str "[ \t\n]+") str))
(defun ee-split-and-expand (str) (mapcar 'ee-expand (ee-split str)))

(defun find-comintprocess-ne (name program-and-args)
  (let ((argv (ee-split program-and-args)))
    (apply 'make-comint name (car argv) nil (cdr argv))
    (switch-to-buffer (format "*%s*" name))))

(defun find-comintprocess (name program-and-args)
  (find-comintprocess-ne   name (ee-split-and-expand program-and-args)))

(defun eepitch-comint (name program-and-args)
"Set `eepitch' to run PROGRAM-AND-ARGS in comint mode, in the buffer \"*NAME*\"."
  (eepitch `(find-comintprocess ,name ',program-and-args)))




;;;        _             _         
;;;   __ _| |_   _ _ __ | |__  ___ 
;;;  / _` | | | | | '_ \| '_ \/ __|
;;; | (_| | | |_| | |_) | | | \__ \
;;;  \__, |_|\__, | .__/|_| |_|___/
;;;  |___/   |___/|_|              
;;;
;; (find-eev "eev-glyphs.el")
;; (find-eevarticlesection "glyphs")

(defface eepitch-star-face
  '((t (:foreground "red")))
  "Face used for the red star glyph (char 15).")

(defun eepitch-set-glyph (pos &optional char face)
  (aset standard-display-table pos
	(if char (vector (make-glyph-code char face)))))




;;;                           
;;; __      ___ __ __ _ _ __  
;;; \ \ /\ / / '__/ _` | '_ \ 
;;;  \ V  V /| | | (_| | |_) |
;;;   \_/\_/ |_|  \__,_| .__/ 
;;;                    |_|    
;;
;; (find-eev "eev-insert.el")

(defun ee-no-properties (str)
  (setq str (copy-sequence str))
  (set-text-properties 0 (length str) nil str)
  str)

(defun eepitch-delete-and-extract-line ()
  (delete-and-extract-region (ee-bol) (ee-eol)))

(defun eepitch-wrap () (interactive)
  (let* ((fmt   " (eepitch-%s)\n (eepitch-kill)\n (eepitch-%s)")
	 (li    (eepitch-delete-and-extract-line))
	 (newli (format fmt li li)))
    (insert newli))
  (ee-next-line 1))



;;;           _           _             _         
;;;  ___  ___| |_    __ _| |_   _ _ __ | |__  ___ 
;;; / __|/ _ \ __|  / _` | | | | | '_ \| '_ \/ __|
;;; \__ \  __/ |_  | (_| | | |_| | |_) | | | \__ \
;;; |___/\___|\__|  \__, |_|\__, | .__/|_| |_|___/
;;;                 |___/   |___/|_|              

(if (not standard-display-table)
    (setq standard-display-table (make-display-table)))
(eepitch-set-glyph ?\^O ?* 'eepitch-star-face)


;;;           _     _                  
;;;  ___  ___| |_  | | _____ _   _ ___ 
;;; / __|/ _ \ __| | |/ / _ \ | | / __|
;;; \__ \  __/ |_  |   <  __/ |_| \__ \
;;; |___/\___|\__| |_|\_\___|\__, |___/
;;;                          |___/     

(global-set-key [f8]   'eepitch-this-line)
(global-set-key "\M-T" 'eepitch-wrap)

(provide 'eepitch)


;;;  _____           _          __   _   _                               
;;; | ____|_ __   __| |   ___  / _| | |_| |__   ___    ___ ___  _ __ ___ 
;;; |  _| | '_ \ / _` |  / _ \| |_  | __| '_ \ / _ \  / __/ _ \| '__/ _ \
;;; | |___| | | | (_| | | (_) |  _| | |_| | | |  __/ | (_| (_) | | |  __/
;;; |_____|_| |_|\__,_|  \___/|_|    \__|_| |_|\___|  \___\___/|_|  \___|
;;;                                                                      
;;; ----------------------------------------------------------------------
;;; ----------------------------------------------------------------------
;;; ----------------------------------------------------------------------




;;;                      _       _                  _           
;;;   ___ ___  _ __ ___ (_)_ __ | |_       ___  ___| |__   ___  
;;;  / __/ _ \| '_ ` _ \| | '_ \| __|____ / _ \/ __| '_ \ / _ \ 
;;; | (_| (_) | | | | | | | | | | ||_____|  __/ (__| | | | (_) |
;;;  \___\___/|_| |_| |_|_|_| |_|\__|     \___|\___|_| |_|\___/ 
;;;                                                             
;; What is this: I am trying to find an elegant way to deal with
;; programs that echo their input (like zsh)... This is still a bit
;; experimental!
;; See: (find-variable 'comint-process-echoes)
;; To do: send an e-mail to Olin Shivers about echoing and stty.

(defun at-eepitch-target (code)
  (eepitch-prepare)
  (save-selected-window
    (select-window (eepitch-target-window))
    (eval code)))

(defun del-echo (flag)
"A hack to help determining whether a program echoes its commands or not.
An example of use:\n
 (eepitch-zsh)
 (eepitch-kill)
 (eepitch-zsh)
cd /tmp/
 (del-echo t)
cd /tmp/
 (del-echo nil)
cd /tmp/\n"
  (at-eepitch-target `(setq comint-process-echoes ,flag))
  (message "At %s: %S" eepitch-buffer-name
	   `(setq comint-process-echoes ,flag)))

(defun eepitch-de (code)
  "Like `eepitch', but deletes the echoed commands.
Use this to control programs that echo the commands that they receive."
  (eepitch `(progn ,code (setq comint-process-echoes t))))

(defun eepitch-comint-de (name program-and-args)
  "Like `eepitch-comint', but deletes the echoed commands.
Use this to control programs that echo the commands that they receive."
  (eepitch-de `(find-comintprocess ,name ',program-and-args)))



;;;   ___  _   _                 _              _     
;;;  / _ \| |_| |__   ___ _ __  | |_ ___   ___ | |___ 
;;; | | | | __| '_ \ / _ \ '__| | __/ _ \ / _ \| / __|
;;; | |_| | |_| | | |  __/ |    | || (_) | (_) | \__ \
;;;  \___/ \__|_| |_|\___|_|     \__\___/ \___/|_|___/
;;;                                                   

(defun ee-at0 (dir code)
  "Eval CODE at DIR.
If DIR does not end with a slash then weird things might happen.
Note the DIR is `ee-expand'-ed."
  (let ((default-directory (ee-expand dir)))
    (if (not (file-accessible-directory-p dir))
	(error "Can't chdir to %s" dir))
    (eval code)))

(defun eepitch-comint-at (dir name program-and-args)
  (ee-at0 dir `(eepitch-comint ,name ,program-and-args)))

(defun with-pager-cat (code)
  (let ((process-environment (cons "PAGER=cat" process-environment)))
    (eval code)))

(defun eepitch-to-buffer (name) (interactive "beepitch to buffer: ")
  (eepitch `(switch-to-buffer ,name)))

(defun at-nth-window (n code)
  "Run `other-window' N times, run CODE there, and go back."
  (save-selected-window
    (other-window n)
    (eval code)))



;;;  _                                                  
;;; | |    __ _ _ __   __ _ _   _  __ _  __ _  ___  ___ 
;;; | |   / _` | '_ \ / _` | | | |/ _` |/ _` |/ _ \/ __|
;;; | |__| (_| | | | | (_| | |_| | (_| | (_| |  __/\__ \
;;; |_____\__,_|_| |_|\__, |\__,_|\__,_|\__, |\___||___/
;;;                   |___/             |___/           

;; Shells
(defun eepitch-shell  () (interactive) (eepitch '(shell)))
(defun eepitch-shell2 () (interactive) (eepitch '(shell "*shell 2*")))
(defun eepitch-eshell () (interactive) (eepitch '(eshell)))
(defun eepitch-bash () (interactive) (eepitch-comint "bash" "bash"))
(defun eepitch-dash () (interactive) (eepitch-comint "dash" "dash"))
(defun eepitch-ksh  () (interactive) (eepitch-comint "ksh" "ksh"))
(defun eepitch-tcsh () (interactive) (eepitch-comint "tcsh" "tcsh"))
(defun eepitch-zsh  () (interactive) (eepitch-comint-de "zsh" "zsh"))
(defun eepitch-scsh () (interactive) (eepitch-comint "scsh" "scsh"))

;; Main interpreted languages
(defun eepitch-lua51  () (interactive) (eepitch-comint "lua51"  "lua5.1"))
(defun eepitch-python () (interactive) (eepitch-comint "python" "python"))
(defun eepitch-ruby   () (interactive) (eepitch-comint "ruby"   "irb1.8"))
(defun eepitch-perl () (interactive) (eepitch-comint "perl" "perl -d -e 42"))

;; Tcl
(defun eepitch-tcl     () (interactive) (eepitch-comint "tclsh"   "tclsh"))
(defun eepitch-tclsh   () (interactive) (eepitch-comint "tclsh"   "tclsh"))
(defun eepitch-wish    () (interactive) (eepitch-comint "wish"    "wish"))
(defun eepitch-expect  () (interactive) (eepitch-comint "expect"  "expect"))

;; Lisps and Schemes
;; It would be better to run them in Slime.
(defun eepitch-sbcl   () (interactive) (eepitch-comint "sbcl" "sbcl"))
(defun eepitch-gcl    () (interactive) (eepitch-comint "gcl"  "gcl"))
(defun eepitch-guile  () (interactive) (eepitch-comint "guile" "guile"))
(defun eepitch-mitscheme () (interactive)
  (eepitch-comint "mit-scheme" "mit-scheme"))

;; Haskell, ML, Erlang, Coq
(defun eepitch-hugs   () (interactive) (eepitch-comint "hugs" "hugs"))
(defun eepitch-hugs98 () (interactive) (eepitch-comint "hugs" "hugs -98"))
(defun eepitch-ghci   () (interactive) (eepitch-comint "ghci" "ghci"))
(defun eepitch-ocaml  () (interactive) (eepitch-comint "ocaml" "ocaml"))
(defun eepitch-labltk () (interactive) (eepitch-comint "labltk" "labltk"))
(defun eepitch-polyml () (interactive) (eepitch-comint "polyml" "poly"))
(defun eepitch-erl    () (interactive) (eepitch-comint "erl" "erl"))
(defun eepitch-coqtop () (interactive) (eepitch-comint "coqtop" "coqtop"))

;; Forth
(defun eepitch-gforth () (interactive) (eepitch '(run-forth "gforth")))
(defun eepitch-gforth () (interactive) (eepitch-comint "gforth" "gforth"))
(defun eepitch-pforth () (interactive) (eepitch-comint "pforth" "pforth"))
(defun eepitch-yforth () (interactive) (eepitch-comint "yforth" "yforth"))

;; Mathematics
(defun eepitch-maxima () (interactive) (eepitch-comint "maxima" "maxima"))
(defun eepitch-octave () (interactive) (eepitch-comint "octave" "octave"))
(defun eepitch-R () (interactive)
  (eepitch '(with-pager-cat (find-comintprocess "R" "R"))))

;; Plotters.
;; We force GhostScript's resolution to make its window fit on the screen.
(defun eepitch-gs () (interactive) (eepitch-comint "gs" "gs -r45"))
(defun eepitch-gs () (interactive) (eepitch-comint "gs" "gs -r60"))
(defun eepitch-gnuplot () (interactive) (eepitch-comint "gnuplot" "gnuplot"))

;; Java-based languages
(defun eepitch-bsh () (interactive)
  (eepitch-de '(find-comintprocess "bsh" "bsh")))
(defun eepitch-scala () (interactive)
  (eepitch '(find-comintprocess "scala" "scala")))
(defun eepitch-clojure () (interactive)
  (eepitch '(find-comintprocess "clojure" "clojure -r")))

;; SQL. To do: add postgres and sqlite
(defun eepitch-mysql () (interactive)
  (eepitch '(with-pager-cat '(find-comintprocess "mysql" "mysql -u root"))))

;; A Javascript REPL in a running Mozilla browser.
;; See: https://github.com/bard/mozrepl/wiki/tutorial
(defun eepitch-mozrepl () (interactive)
  (eepitch-comint "mozrepl" "telnet localhost 4242"))

;; Programs from the TeX family.
;; They create logfiles in the current dir, so we run them in /tmp/.
(defun eepitch-luatex () (interactive)
  (eepitch-comint-at "/tmp/" "luatex" "luatex"))
(defun eepitch-lualatex () (interactive)
  (eepitch-comint-at "/tmp/" "lualatex" "lualatex"))
(defun eepitch-latex () (interactive)
  (eepitch-comint-at "/tmp/" "latex" "latex"))
(defun eepitch-tex   () (interactive)
  (eepitch-comint-at "/tmp/" "tex"   "tex"))
(defun eepitch-mf    () (interactive)
  (eepitch-comint-at "/tmp/" "mf"   "mf"))
(defun eepitch-mpost () (interactive)
  (eepitch-comint-at "/tmp/" "mpost" "mpost"))

;; Pulseaudio (this is to interact with its daemon)
(defun eepitch-pacmd () (interactive) (eepitch-comint "pacmd" "pacmd"))




;;;           _          
;;;  _ __ ___(_)_ __ ___ 
;;; | '__/ __| | '__/ __|
;;; | | | (__| | | | (__ 
;;; |_|  \___|_|_|  \___|
;;;                      

(defun ee-rcirc-connected ()
  (and (get-buffer           "*irc.freenode.net*")
       (rcirc-buffer-process "*irc.freenode.net*")))

(defun ee-rcirc-connect ()
  "Connect to freenode if not already connected."
  (if (not (ee-rcirc-connected))
      (rcirc-connect "irc.freenode.net" nil nil nil nil '("#eev")))
  (switch-to-buffer "*irc.freenode.net*"))

(defun eepitch-freenode () (interactive)
  (eepitch '(ee-rcirc-connect))
  ;; Override the the default behavior of eepitch-kill:
  (setq eepitch-kill '(message "Not killing %S" "*irc.freenode.net*")))




;;;            _ _     
;;;   __ _  __| | |__  
;;;  / _` |/ _` | '_ \ 
;;; | (_| | (_| | |_) |
;;;  \__, |\__,_|_.__/ 
;;;  |___/             
;;
;; I AM NOT HAPPY WITH THIS! 8-(
;; This is not running well at Marc's machine...
;; This is the most experimental part of eepitch, and will certainly
;; change a lot. Note that Slime also uses several buffers - the
;; lessons learned here will probably also apply to controlling Slime.

(defvar eepitch-gdb-buffer-name nil)
(defvar eepitch-i/o-buffer-name nil)

(defun eepitch-window-show-3 ()
  "Display the buffer `eepitch-buffer-name' in another window.
This function uses a hardcoded 3-window setup, intended mainly for gdb."
  (delete-other-windows)
  (split-window-horizontally)  ; left|right
  (split-window-vertically)    ; split the left side into upper/lower
  (at-nth-window 1 '(switch-to-buffer eepitch-buffer-name)))

(defun eepitch-window-show-4 ()
  "Display the buffer `eepitch-buffer-name' in another window.
This function uses a hardcoded 4-window setup, intended mainly for gdb."
  (delete-other-windows)
  (split-window-horizontally)  ; left|right
  (split-window-vertically)    ; split the left side into upper/lower
  (at-nth-window 2 '(split-window-vertically)) ; split the right side into u/l
  (at-nth-window 1 '(switch-to-buffer eepitch-buffer-name))
  (at-nth-window 3 '(switch-to-buffer eepitch-i/o-buffer-name)))

;; To do: reimplement gud-keys (M-n for "next", etc) here.
;; (find-eev "eev-mini-steps.el" "eegud-kill-this-buffer")
;; (find-eev "eev-langs.el" nil "define-minor-mode eegud-keys-mode")
;; (setq eepitch-code `(progn (gdb ,cmd) (eegud-keys-mode 1)))

(defun eepitch-gdb-buffer-name-setup (program)
  (let (stem (file-name-nondirectory fullfname))
    (setq eepitch-gdb-buffer-name (format "*gud-%s*" stem))
    (setq eepitch-i/o-buffer-name (format "*input/output of %s*" stem))
    (setq eepitch-buffer-name eepitch-gdb-buffer-name)))

(defun eepitch-gdb-core-setup (gdb fmt program)
  (setq eepitch-code (list gdb (format fmt (ee-expand program))))
  (eepitch-gdb-buffer-name-setup program)
  (eepitch-prepare))

(defun eepitch-gdb-3 (program)
  (setq eepitch-window-show '(eepitch-window-show-3))
  (eepitch-gdb-core-setup 'gud-gdb "gdb --fullname %s" program))

(defun eepitch-gdb-4 (program)
  (setq eepitch-window-show '(eepitch-window-show-4))
  (eepitch-gdb-core-setup 'gdb "gdb --annotate=3 %s" program))

(defun eepitch-gdb-4 (program)
  (setq eepitch-window-show '(eepitch-window-show-4))
  (eepitch-gdb-core-setup 'gdb "gdb -i=mi %s" program))



;; The tests for the GDB interface have been moved to:
;; (find-eev "eepitch.readme" "gdb-preparation")
;; (find-eev "eepitch.readme" "gdb-3-windows")
;; (find-eev "eepitch.readme" "gdb-4-windows")






;; Local Variables:
;; coding:            raw-text-unix
;; ee-anchor-format:  "defun %s "
;; no-byte-compile:   t
;; End:
